/*
 * @(#)ButtonProperties.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.button;

import java.beans.*;
import java.io.*;

import com.wirefusion.project.*;
import com.wirefusion.propertyeditor.*;
import com.wirefusion.undo.*;

public class ButtonProperties extends WobProperties implements Externalizable,
		PropertyEditorCollectionFactory {
	/**
	 * Unique "random" ID for the ButtonWob class. Used by externalization
	 * mechanism.
	 */
	static final long serialVersionUID = 26613164088814673L;

	/**
	 * The selected button mode (BASIC_MODE or ADVANCED_MODE). Default mode is
	 * BASIC_MODE.
	 */
	private int mode = BASIC_MODE;

	/**
	 * The current label. Only used in basic mode.
	 */
	private String basicLabel = Messages.getString("ButtonProperties.Button");

	/**
	 * Stores data about the button configurations. Only used in Advanced mode.
	 * stateData[UP] stores button configuration related to the UP button state,
	 * stateData[DOWN] for DOWN state...
	 */
	private ButtonStateData[] stateData = new ButtonStateData[4];

	/**
	 * The repeat rate. Specified as button clicks per second.
	 */
	private double repeatRate = 0;

	/**
	 * The UP state
	 */
	public static final int UP_STATE = 0;

	/**
	 * The DOWN state
	 */
	public static final int DOWN_STATE = 1;

	/**
	 * The OVER state
	 */
	public static final int OVER_STATE = 2;

	/**
	 * The DISABLED state
	 */
	public static final int DISABLED_STATE = 3;

	/**
	 * Basic mode
	 */
	public static final int BASIC_MODE = 0;

	/**
	 * Advanced mode
	 */
	public static final int ADVANCED_MODE = 1;

	/**
	 * Constructs a new ButtonProperties object.
	 */
	public ButtonProperties() {
		// Initializes the button state data (used in Advanced mode)
		stateData[UP_STATE] = new ButtonStateData(this, true, "" + (UP_STATE + 1), "" + (UP_STATE + 10));
		stateData[DOWN_STATE] = new ButtonStateData(this, false, "" + (DOWN_STATE + 1), "" + (DOWN_STATE + 10));
		stateData[OVER_STATE] = new ButtonStateData(this, false, "" + (OVER_STATE + 1), "" + (OVER_STATE + 10));
		stateData[DISABLED_STATE] = new ButtonStateData(this, false, "" + (DISABLED_STATE + 1), "" + (DISABLED_STATE + 10));
	}

	/**
	 * Returns the current button mode
	 * 
	 * @return the current button mode
	 */
	public int getMode() {
		return mode;
	}

	/**
	 * Sets the current button mode (BASIC_MODE or ADVANCED_MODE)
	 * 
	 * @param mode
	 *          the mode
	 */
	public void setMode(int mode) {
		this.mode = mode;
	}

	/**
	 * Returns button state data for the specified state
	 * 
	 * @param buttonState
	 * @return
	 */
	public ButtonStateData getStateData(int buttonState) {
		return stateData[buttonState];
	}

	/**
	 * Returns the file of the button image (in UP state). Used in
	 * Advanced mode only.
	 * 
	 * @return the filename of the default button image (in UP state). Used in
	 *         Advanced mode only.
	 */
	public File getButtonUpImageFile() {
		return stateData[UP_STATE].getButtonImageFile();
	}

	/**
	 * Returns the file of the label image (in UP state). Used in
	 * Advanced mode only.
	 * 
	 * @return the file of the default button image (in UP state). Used in
	 *         Advanced mode only.
	 */
	public File getLabelUpImageFile() {
		return stateData[UP_STATE].getLabelImageFile();
	}

	/**
	 * Returns the label string. Used in Basic mode only.
	 * 
	 * @return the label string.
	 */
	public String getLabel() {
		return basicLabel;
	}

	/**
	 * Sets the label string. Used in Advanced mode only.
	 * 
	 * @param label
	 *          the label string
	 */
	public void setLabel(String label) {
		this.basicLabel = label;
	}

	/**
	 * Reads the state of a previously stored ButtonState object.
	 */
	public void writeExternal(ObjectOutput objectoutput) {
		try {
			super.writeExternal(objectoutput);
			objectoutput.writeUTF(basicLabel);
			objectoutput.writeDouble(repeatRate);
			objectoutput.writeInt(mode);
			for (int i = 0; i < 4; i++) {
				objectoutput.writeObject(stateData[i]);
			}
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

	/**
	 * Saves the state of the ButtonState object.
	 */
	public void readExternal(ObjectInput objectinput) {
		try {
			super.readExternal(objectinput);
			basicLabel = objectinput.readUTF();

			if (getSerializedWobVersion() >= 2.0) {
				repeatRate = objectinput.readDouble();
				mode = objectinput.readInt();
				for (int i = 0; i < 4; i++) {
					stateData[i] = (ButtonStateData) objectinput.readObject();
				}
			}
		} catch (Exception exception) {
			if (!exception.getMessage().equals("progress"))
				exception.printStackTrace();
		}
	}

	/**
	 * Writes data to the player data file, which will later be read by the
	 * ButtonWob.readData() method.
	 */
	public void writeData(DataOutputStream dos) {
		try {
			dos.writeDouble(repeatRate);
			dos.writeBoolean(mode == BASIC_MODE);
			if (mode == BASIC_MODE) {
				dos.writeUTF(basicLabel);
			} else {
				for (int buttonMode = 0; buttonMode < 4; buttonMode++) {
					File buttonImageFile = stateData[buttonMode].getButtonImageFile();
					if (buttonImageFile != null) {
						dos.writeUTF(buttonImageFile.getName());
					} else {
						dos.writeUTF("");
					}
					
					File buttonLabelFile = stateData[buttonMode].getLabelImageFile();
					if (buttonLabelFile != null) {
						dos.writeUTF(buttonLabelFile.getName());
					} else {
						dos.writeUTF("");
					}
				}
			}
		} catch (IOException ioexception) {
			ioexception.printStackTrace();
		}
	}

	/**
	 * Sets the repeat rate (button clicks/sec)
	 * 
	 * @param repeatRate
	 *          the repeat rate (button clicks/sec)
	 */
	public void setRepeatRate(double repeatRate) {
		if (this.repeatRate != repeatRate) {
			UndoableDoubleEdit edit = new UndoableDoubleEdit(this.repeatRate,
					repeatRate) {
				public void setValue(double value) {
					ButtonProperties.this.setRepeatRate(value);
				}
			};
			getUndoManager().addEdit(edit);

			this.repeatRate = repeatRate;
			firePropertyChange("repeatRateChanged", null, new Double(repeatRate));
		}
	}

	/**
	 * Returns the repeat rate
	 * 
	 * @return the repeat rate
	 */
	public double getRepeatRate() {
		return repeatRate;
	}

	/**
	 * Creates and returns the property editor collection, which is used by
	 * WireFusion to present the input for this Wob in the Editor Pane.
	 */
	public PropertyEditorCollection createPropertyEditorCollection() {
		PropertyEditorCollection propertyEditorCollection = new PropertyEditorCollection(
				this);

		// The repeat rate property
		NumberPropertyEditor valueEditor = new NumberPropertyEditor(
				"repeatRateChanged",
				Messages.getString("ButtonProperties.Repeat_rate"), new Double(
						getRepeatRate()));
		valueEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				double repeatRate = ((Double) evt.getNewValue()).doubleValue();
				setRepeatRate(repeatRate);
			}
		});
		propertyEditorCollection.addPropertyEditor(valueEditor);

		addPropertyChangeListener(propertyEditorCollection);
		return propertyEditorCollection;
	}

}
