/*
 * @(#)ButtonPropertiesPanel.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.button;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import com.wirefusion.gui.*;
import com.wirefusion.project.*;
import com.wirefusion.utils.*;

/**
 * The GUI of the button dialog window.
 */
public class ButtonPropertiesPanel extends WobPropertiesPanel {

	/**
	 * The label text field used when in Basic mode.
	 */
	private JTextField basicLabelTextField;

	/**
	 * A tabbed pane, used in Advanced mode. Each tab contains a
	 * ButtonStateConfigurator object for a specific state.
	 */
	private JTabbedPane tabbedPane;

	/**
	 * Checkbox used for switching between Advanced and Basic mode.
	 */
	private JCheckBox advancedModeCheckbox;

	/**
	 * Reference back to the Button properties object
	 */
	private ButtonProperties buttonProperties;

	/**
	 * Holds button state configurators. Used in Advanced mode only.
	 */
	private ButtonStateConfigurator[] configurators = new ButtonStateConfigurator[4];

	/**
	 * Initializes the dialog GUI
	 */
	public void init(WobProperties wobProperties) {
		this.buttonProperties = (ButtonProperties) wobProperties;
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		advancedModeCheckbox = new JCheckBox(Messages
				.getString("ButtonPropertiesPanel.Advanced_Mode"), buttonProperties
				.getMode() == ButtonProperties.ADVANCED_MODE);
		advancedModeCheckbox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				JDialog parentDialog = getDialog();
				updateLayout();
				parentDialog.pack();
				parentDialog.setSize(parentDialog.getPreferredSize());
			}
		});

		basicLabelTextField = new JTextField(20);
		basicLabelTextField.setText(buttonProperties.getLabel());

		// Get default button image files and initialize the button configurators
		File defaultButtonImageFile = buttonProperties
				.getFileInWobFolder("defaultbutton.jpg");
		configurators[ButtonProperties.UP_STATE] = new ButtonStateConfigurator(
				buttonProperties.getStateData(ButtonProperties.UP_STATE), this,
				Messages.getString("ButtonPropertiesPanel.Up"), defaultButtonImageFile,
				true);

		defaultButtonImageFile = buttonProperties
				.getFileInWobFolder("downbutton.jpg");
		configurators[ButtonProperties.DOWN_STATE] = new ButtonStateConfigurator(
				buttonProperties.getStateData(ButtonProperties.DOWN_STATE), this,
				Messages.getString("ButtonPropertiesPanel.Down"),
				defaultButtonImageFile, false);

		defaultButtonImageFile = buttonProperties
				.getFileInWobFolder("overbutton.jpg");
		configurators[ButtonProperties.OVER_STATE] = new ButtonStateConfigurator(
				buttonProperties.getStateData(ButtonProperties.OVER_STATE), this,
				Messages.getString("ButtonPropertiesPanel.Over"),
				defaultButtonImageFile, false);

		defaultButtonImageFile = buttonProperties
				.getFileInWobFolder("disabledbutton.jpg");
		configurators[ButtonProperties.DISABLED_STATE] = new ButtonStateConfigurator(
				buttonProperties.getStateData(ButtonProperties.DISABLED_STATE), this,
				Messages.getString("ButtonPropertiesPanel.Disabled"),
				defaultButtonImageFile, false);

		// Create the tabbed pane that should hold the ButtonStateConfigurator
		// objects
		tabbedPane = new JTabbedPane() {
			public Dimension getPreferredSize() {
				int width = super.getPreferredSize().width;
				width = Math.max(width, 370);
				return new Dimension(width, super.getPreferredSize().height);
			}
		};

		for (int i = 0; i < configurators.length; i++) {
			tabbedPane.add(configurators[i], configurators[i].getID());
		}

		updateLayout();
	}

	public ButtonProperties getButtonProperties() {
		return buttonProperties;
	}

	public ButtonStateConfigurator[] getConfigurators() {
		return configurators;
	}

	private void updateLayout() {
		removeAll();
		GridBagLayout gridbaglayout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(gridbaglayout);
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.insets = new Insets(3, 5, 3, 5);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.anchor = GridBagConstraints.WEST;
		add(advancedModeCheckbox, gbc);

		if (!isAdvancedMode()) {
			gbc.gridwidth = 1;
			gbc.gridx = 0;
			gbc.gridy = 1;
			gbc.insets = new Insets(20, 5, 10, 5);
			gbc.anchor = GridBagConstraints.EAST;
			JLabel basicLabelLabel = new JLabel(Messages
					.getString("ButtonPropertiesPanel.Label"));
			add(basicLabelLabel, gbc);

			gbc.gridx = 1;
			gbc.weightx = 10000;
			gbc.anchor = GridBagConstraints.WEST;
			add(basicLabelTextField, gbc);
			gbc.weightx = 1;
		} else {
			gbc.gridwidth = 2;
			gbc.gridx = 0;
			gbc.gridy = 1;
			gbc.insets = new Insets(20, 5, 10, 5);
			gbc.fill = GridBagConstraints.HORIZONTAL;
			add(tabbedPane, gbc);
		}
	}

	public boolean isAdvancedMode() {
		return advancedModeCheckbox.isSelected();
	}

	public void destroy() {
		WobPort setLabelPort = buttonProperties.getPort("set_label");
		// If Basic, make sure the "set Label" in-port exists
		if (!isAdvancedMode()) {
			if (setLabelPort == null) {
				buttonProperties.createPort("set_label");
			}
			// If Advanced mode and the "set Label" port exists, remove the port
		} else { // Image button
			if (setLabelPort != null) {
				if (setLabelPort.isConnected()) {
					JOptionPane
							.showMessageDialog(
									this,
									Messages
											.getString("ButtonPropertiesPanel.Wires_connected_to_this_Object_has_been_removed"),
									"", JOptionPane.INFORMATION_MESSAGE);
				}
				setLabelPort.delete();
			}
		}
	}

	public void apply() {
		buttonProperties
				.setMode(advancedModeCheckbox.isSelected() ? ButtonProperties.ADVANCED_MODE
						: ButtonProperties.BASIC_MODE);

		// Save advanced mode configurators
		for (int i = 0; i < configurators.length; i++) {
			configurators[i].applyConfiguration();
		}

		if (isAdvancedMode()) {
			// Set the size of the Target Area
			int maxWidth = 0;
			int maxHeight = 0;
			for (int i = 0; i < configurators.length; i++) {
				maxWidth = Math.max(configurators[i].getButtonSize().width, maxWidth);
				maxHeight = Math
						.max(configurators[i].getButtonSize().height, maxHeight);
			}
			buttonProperties.getTargetArea().setSize(
					new Dimension(maxWidth, maxHeight));
		} else {
			buttonProperties.setLabel(basicLabelTextField.getText().trim());
		}
	}

}