/*
 * @(#)ButtonStateConfigurator.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.button;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import com.wirefusion.gui.misc.*;
import com.wirefusion.gui.textimage.*;
import com.wirefusion.project.*;
import com.wirefusion.utils.*;

/**
 * A Button Mode configurators contains the GUI used for configuring a button
 * state (UP, DOWN, OVER or DISABLED) in the Advanced mode. Each state has one
 * corresponding ButtonStateConfigurator object.
 */
public class ButtonStateConfigurator extends JPanel implements ActionListener {
	/**
	 * Stores the current configurator of the button state
	 */
	private ButtonStateData buttonStateData;

	/**
	 * Clicking this button opens an image file dialog, where the button image for
	 * the state can be loaded.
	 */
	private JButton changeButtonImageButton;

	/**
	 * Clicking this button opens a TextImage editor, where one can specify the
	 * button label text and its apperance that should be used as button label for
	 * the state.
	 */
	private JButton changeLabelImageButton;

	/**
	 * This checkbox is used for enabling and disabling the configuration of
	 * graphics for the state.
	 */
	private JCheckBox enableCheckBox;

	/**
	 * The currently selected button image
	 */
	private File buttonImageFile;

	/**
	 * The currently label image
	 */
	private File labelImageFile;

	/**
	 * A clone of the current text image data is used during configuration. It is
	 * permanented when the user clicks OK or Apply.
	 */
	private TextImage tmpTextImageData;

	/**
	 * Preview of the current button configuration in this state
	 */
	private ButtonPreview buttonPreview;

	/**
	 * A reference back to the button properties panel obejct.
	 */
	private ButtonPropertiesPanel buttonPropertiesPanel;

	private String idString;

	/**
	 * Creates a ButtonStateConfigurator object for a specified button state.
	 * 
	 * @param buttonStateData
	 *          The initial button state data
	 * @param buttonPropertiesPanel
	 *          a reference back to the button properties panel
	 * @param idString
	 *          a string unique for the button state that this object is editor
	 *          for.
	 * @param defaultIconFile
	 *          the default icon file.
	 * @param alwaysEnabled
	 *          Specifies if this state can be disabled.
	 */
	public ButtonStateConfigurator(ButtonStateData buttonStateData,
			ButtonPropertiesPanel buttonPropertiesPanel, String idString,
			File defaultIconFile, boolean alwaysEnabled) {
		this.buttonStateData = buttonStateData;
		this.buttonPropertiesPanel = buttonPropertiesPanel;
		this.idString = idString;

		// Use a clone of the label image configuration. The original should not be
		// used, since the user may cancel the dialog...
		if (buttonStateData.getLabelImageConfig() != null)
			tmpTextImageData = buttonStateData.getLabelImageConfig().getClone();

		if (buttonStateData.getButtonImageFile() == null) {
			buttonImageFile = defaultIconFile;
		} else {
			buttonImageFile = buttonStateData.getButtonImageFile();
		}

		if (buttonStateData.getLabelImageFile() != null)
			labelImageFile = buttonStateData.getLabelImageFile();

		enableCheckBox = new JCheckBox(Messages
				.getString("ButtonStateConfigurator.Enable"), buttonStateData
				.isEnabled());
		enableCheckBox.setActionCommand("enableChange");
		enableCheckBox.addActionListener(this);

		changeButtonImageButton = new JButton(Messages
				.getString("ButtonStateConfigurator.Change_Graphics..."));
		changeButtonImageButton.setActionCommand("changeIcon");
		changeButtonImageButton.addActionListener(this);

		changeLabelImageButton = new JButton(Messages
				.getString("ButtonStateConfigurator.Change_Label..."));
		changeLabelImageButton.setActionCommand("changeLabel");
		changeLabelImageButton.addActionListener(this);

		buttonPreview = new ButtonPreview(this);
		buttonPreview.updatePreview();

		setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.insets = new Insets(3, 5, 3, 5);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		if (alwaysEnabled) {
			// Just add a dummycomponent, with equal size to the checkboxes,
			// to get equal layout for all GUI panels
			JPanel dummyPanel = new JPanel();
			dummyPanel.setPreferredSize(enableCheckBox.getPreferredSize());
			add(dummyPanel, gbc);
		} else {
			add(enableCheckBox, gbc);
		}

		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		add(changeButtonImageButton, gbc);

		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.weightx = 1000000;
		add(changeLabelImageButton, gbc);

		gbc.gridheight = 3;
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.fill = GridBagConstraints.BOTH;
		JPanel p = new JPanel();
		p.setMinimumSize(new Dimension(100, 80));
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), Messages
				.getString("ButtonStateConfigurator.Preview")));
		p.add(buttonPreview);
		add(p, gbc);
		updateEnablings();
	}

	public String getID() {
		return idString;
	}

	public Dimension getButtonSize() {
		if (enableCheckBox.isSelected())
			return buttonPreview.getButtonDimension();
		else
			return new Dimension(0, 0);
	}

	public void applyConfiguration() {
		boolean activeMode = buttonPropertiesPanel.isAdvancedMode();
		buttonStateData.setButtonImageFile(buttonImageFile);

		buttonStateData.setEnabled(enableCheckBox.isSelected());
		buttonStateData.setLabelImageConfig(tmpTextImageData);

		if (enableCheckBox.isSelected() && activeMode) {
			buttonStateData.setButtonImageFile(buttonImageFile);

			if (labelImageFile != null
					&& !tmpTextImageData.getPlainText().trim().equals(
							"")) {
				buttonStateData.setLabelImageFile(labelImageFile);
				buttonStateData.setLabelImageConfig(tmpTextImageData.getClone());
			} else {
				buttonStateData.setLabelImageFile(null);
				buttonStateData.setLabelImageConfig(null);
			}
		} else {
			buttonStateData.setButtonImageFile(null);
			buttonStateData.setLabelImageFile(null);
			buttonStateData.setLabelImageConfig(null);
		}
		buttonStateData.setEnabled(enableCheckBox.isSelected());
	}

	private File getImage() {
		// The default folder with id "image" i fetched. (it is the same folder id
		// as used by for example the Image object)
		File defaultFile = WobUtilities.getDefaultFolder("image", new File(
				WobUtilities.USER_RESOURCES_FOLDER, "images"));
		ImageFileChooser chooser = new ImageFileChooser(defaultFile);
		chooser.setImageFileFilters(new String[] { "gif", "jpg", "png" });
		JDialog dialog = (JDialog) SwingUtilities.getRoot(this);
		int returnVal = chooser.showOpenDialog(dialog);
		// The folder containing the opened file is saved as the new default folder
		// with id "image".
		WobUtilities.setDefaultFolder("image", chooser.getSelectedFile());
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			return chooser.getSelectedFile();
		} else {
			return null;
		}
	}

	private void updateEnablings() {
		boolean enabled = enableCheckBox.isSelected();
		changeButtonImageButton.setEnabled(enabled);
		changeLabelImageButton.setEnabled(enabled);
		buttonPreview.setEnabled(enabled);
	}

	public File getIconFile() {
		return buttonImageFile;
	}

	private void setLabelImageFile(File file) {
		labelImageFile = file;
		buttonPreview.updatePreview();
	}

	public File getLabelImageFile() {
		return labelImageFile;
	}

	public void actionPerformed(ActionEvent ev) {
		JDialog parentDialog = (JDialog) SwingUtilities.getRoot(this);
		if (ev.getActionCommand().startsWith("enableChange")) {
			updateEnablings();
		} else if (ev.getActionCommand().startsWith("changeIcon")) {
			File file = getImage();
			if (file != null) {
				buttonImageFile = file;
				buttonPreview.updatePreview();
				WobUtilities.updateWindowSize(parentDialog);
				validate();
				repaint();
			}
		} else if (ev.getActionCommand().startsWith("changeLabel")) {
			if (tmpTextImageData == null) {
				tmpTextImageData = new TextImage(TextImage.BUTTON_TEXT_MODE);
				tmpTextImageData.setImageName(buttonPropertiesPanel
						.getButtonProperties().getWobName()
						+ "_" + idString.replaceAll(" ", "_"));
			}

			Image img = com.wirefusion.gui.WF.getNonCachedImage(buttonImageFile);
			File file = tmpTextImageData.openEditor(parentDialog, img);

			if (file != null) { // Om ok trycktes
				if (tmpTextImageData.getPlainText().trim().equals(""))
					file = null;

				labelImageFile = file;
				buttonPreview.updatePreview();

				boolean applyToAll = false;
				int type = JOptionPane
						.showConfirmDialog(
								parentDialog,
								Messages
										.getString("ButtonStateConfigurator.Would_you_like_to_use_this_label_for_all_Button_states"),
								"", JOptionPane.YES_NO_OPTION);
				if (type == JOptionPane.OK_OPTION)
					applyToAll = true;

				ButtonStateConfigurator[] configurators = new ButtonStateConfigurator[] { this };
				if (applyToAll)
					configurators = buttonPropertiesPanel.getConfigurators();

				for (int i = 0; i < configurators.length; i++) {
					configurators[i].setLabelImageFile(file);
					configurators[i].tmpTextImageData = tmpTextImageData.getClone(); 
				}

				WobUtilities.updateWindowSize(parentDialog);
				validate();
				repaint();
			}
		}
	}
}