/*
 * @(#)ButtonWob.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.button.player;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import wobs.button.*;
import wobs.button.*;
import com.wirefusion.player.*;

/**
 * <p>
 * This class creates a labeled button. It has two modes, Basic and Advanced. In
 * Advanced mode, the graphics is created from images. In Basic mode all
 * graphics is painted with primitives and drawString. The button is
 * in the DOWN state when it is pressed, otherwise in the UP state.
 * </p>
 * <p>
 * When the user clicks the button, it sends out a "Button Clicked" event. When
 * the user holds the button pressed, and "repeat" is enabled, "Button Clicked"
 * events will be sent repeatedly as long as the button is pressed with the
 * repeat interval specified by repeatPause.
 * </p>
 * <p>
 * If in Basic mode, the button label can be set by sending the new label to an in-port
 * in ButtonWob.
 * </p>
 * <p>
 * If the mouse is pressed and released on the button during a single frame,
 * graphics for DOWN state will still be painted even though the state is UP
 * when it is time to paint. The graphics for UP state will be painted in the
 * next frame.
 * </p>
 */

public class ButtonWob extends Wob {

	/**
	 * Modes: true if Basic Mode, false if Advanced Mode
	 */
	private boolean basicMode;

	/**
	 * Allowed values: ButtonProperties.UP, ButtonProperties.DOWN,
	 * ButtonProperties.OVER, ButtonProperties.DISABLED
	 */
	private int buttonState = ButtonProperties.UP_STATE;

	/**
	 * True if mouse cursor is inside button area
	 */
	private boolean mouseInside = false;

	/**
	 * True if button is pressed
	 */
	private boolean mouseDown = false;

	/**
	 * True if button is disabled
	 */
	private boolean disabled = false;

	/**
	 * The label text. Only used in Basic mode.
	 */
	private String label;

	/**
	 * Light gray color. Only used in Basic mode.
	 */
	private final static Color lightGray = new Color(0xCCCCCC);

	/**
	 * Gray color. Only used in Basic mode.
	 */
	private final static Color gray = new Color(0x999999);

	/**
	 * Dark gray color. Only used in Basic mode.
	 */
	private final static Color darkGray = new Color(0x666666);

	/**
	 * Contains the filenames of the button images. For example,
	 * buttonImageFilenames[ButtonState.DOWN] will contain the image filename for
	 * the DOWN state (or null if down state is disabled). Only used in Advanced
	 * mode.
	 */
	private String[] buttonImageFilenames = new String[4];

	/**
	 * Contains the filenames of the text images. For example,
	 * textImageFilenames[ButtonState.DOWN] will contain the text image filename
	 * for the DOWN state (or null if down state is disabled or text unspecified).
	 * Only used in Advanced mode.
	 */
	private String[] textImageFilenames = new String[4];

	/**
	 * Specifies the repeat rate of "button
	 * clicked" events, when the button is held down. Repeat function is disabled
	 * if value is 0.
	 */
	private double repeatRate;

	/**
	 * Idicates that graphics for ButtonProperties.DOWN mode should be painted in
	 * the next call to paint, even if the button state is not
	 * ButtonProperties.DOWN. Used for forcing button down graphics to be painted
	 * even for quick clicks, where the button is pressed and released within one
	 * frame.
	 */

	
	/**
	 * Indicated that the callMouseRelease flag should be set in the next paint call
	 */
	private boolean callMouseReleaseAfterNextPaint = false;
	
	/**
	 * Indicated that mouseReleaseForGraphics should be called the next time
	 * processPlayerEvent is called (i.e. in the next frame).
	 */
	private boolean callMouseRelease = false;
	
	/**
	 * The last time a mouse press occured on this button was in the frame.
	 */
	private int mousePressedInThisFrame = -1;

	/**
	 * Used by the repeat function. Set to the time of last repeat event sent when
	 * repeat function is in progress.
	 */
	private long timeOfLastRepeat = Long.MAX_VALUE;

	public void init() {
		// Listen to mouse events
		enableEvents(AWTEvent.MOUSE_EVENT_MASK | AWTEvent.COMPONENT_EVENT_MASK);
	}

	
	/**
	 * If the button is held down and repeat mode is enabled, then this object
	 * will listen to FRAME events and send out "Mouse Clicked" events at the
	 * specified rate.
	 */
	public void processPlayerEvent(PlayerEvent ev) {
		// If a FRAME event, send out the number of "Button Clicked" repeats
		// "collected" since the last frame.

		if (callMouseRelease) {			
			mouseRelease();
		}
		if (ev.getID() == PlayerEvent.FRAME_EVENT) {
			// The pause, in milliseconds, between two repeats.
			int repeatPause = (int) (1000 / repeatRate);
			// Calculate number of repeats that should be sent
			long noOfRepeats = (getCore().getPlayerTime() - timeOfLastRepeat)
					/ repeatPause;
			timeOfLastRepeat = timeOfLastRepeat + noOfRepeats * repeatPause;
			for (int i = 0; i < noOfRepeats; i++) {
				sendPulse(1); // Send a "Button Clicked" event
			}
		}
	}

	/**
	 * This method is called after a mouse release event has occured.
	 * If the mouse release event was proceeded by a mouse pressed event,
	 * then mouseRelease() will be delayed for one frame, so that the
	 * proceeding mouse pressed state will also be painted. 
	 */
	private void mouseRelease() {		
			disableEvents(PlayerEvent.PLAYER_EVENT_MASK);
			mouseDown = false;
			repaint();
			callMouseReleaseAfterNextPaint = false;			
			callMouseRelease = false;
	}
	
	/**
	 * Reset to default state if deactivated (prevents strange behaviour 
	 * when deactivated just when button has been pressed and later
	 * activated again)
	 */
	public void processComponentEvent(ComponentEvent ev) {
		if (ev.getID() == ComponentEvent.COMPONENT_HIDDEN) {
			buttonState = ButtonProperties.UP_STATE;
			mouseInside = false;
			mouseDown = false;
			callMouseReleaseAfterNextPaint = false;
			callMouseRelease = false;
			disableEvents(PlayerEvent.PLAYER_EVENT_MASK);
		}
	}
	
	public void processMouseEvent(MouseEvent ev) {
		if (ev.getID() == MouseEvent.MOUSE_PRESSED) {
			enableEvents(PlayerEvent.PLAYER_EVENT_MASK);
			timeOfLastRepeat = getCore().getPlayerTime();
			mouseDown = true;
			callMouseReleaseAfterNextPaint = false;
			callMouseRelease = false;
			mousePressedInThisFrame = getCore().getFrameNumber();
			repaint();
		} else if (ev.getID() == MouseEvent.MOUSE_RELEASED) {
			callMouseRelease = false;
			// If just proceeded by a mouse pressed event, delay
			// the mouse released state for one frame			
			if (mousePressedInThisFrame == getCore().getFrameNumber())
				callMouseReleaseAfterNextPaint = true;
			else
				mouseRelease();
		
			// Send a "Mouse Clicked" event if not disabled
			if (!disabled)
				sendPulse(1);
		} else if (ev.getID() == MouseEvent.MOUSE_ENTERED) {
			mouseInside = true;
			repaint();
		} else if (ev.getID() == MouseEvent.MOUSE_EXITED) {
			mouseInside = false;
			repaint();
		}
	}

	/**
	 * Updates the buttonState field.
	 */
	private void updateButtonMode() {
		if (basicMode) {
			// Basic mode only have DOWN and UP state
			if (mouseDown && !disabled)
				buttonState = ButtonProperties.DOWN_STATE;
			else
				buttonState = ButtonProperties.UP_STATE;
		} else {
			// Advanced mode have the states UP, DOWN, DISABLED, OVER
			if (disabled
					&& !buttonImageFilenames[ButtonProperties.DISABLED_STATE].equals("")) {
				buttonState = ButtonProperties.DISABLED_STATE;
			} else if (!disabled && mouseDown
					&& !buttonImageFilenames[ButtonProperties.DOWN_STATE].equals("")) {
				buttonState = ButtonProperties.DOWN_STATE;
			} else if (!disabled && mouseInside
					&& !buttonImageFilenames[ButtonProperties.OVER_STATE].equals("")) {
				buttonState = ButtonProperties.OVER_STATE;
			} else {
				buttonState = ButtonProperties.UP_STATE;
			}
		}
	}

	/**
	 * Paints the button graphics.
	 */
	public void paint(Graphics graphics) {
		WfGraphics g = (WfGraphics)graphics;
		if (callMouseReleaseAfterNextPaint) {
			callMouseReleaseAfterNextPaint = false;
			callMouseRelease = true;			
		}
		updateButtonMode();

		int w = getBounds().width;
		int h = getBounds().height;
		if (basicMode) {
			g.setAntiAliasing(true);
			if (buttonState == ButtonProperties.DOWN_STATE) {
				g.setColor(lightGray);
				g.fillRect(0, 0, w - 1, h - 1);
				g.setColor(Color.white);
				g.drawRect(1, 1, w - 2, h - 2);
				g.setColor(darkGray);
				g.drawRect(0, 0, w - 2, h - 2);
				g.setColor(gray);
				g.fillRect(1, 1, w - 3, h - 3);
			} else {
				g.setColor(lightGray);
				g.fillRect(0, 0, w - 1, h - 1);
				g.setColor(darkGray);
				g.drawRect(0, 0, w - 2, h - 2);
				g.setColor(Color.white);
				g.drawRect(1, 1, w - 2, h - 2);
			}
			// Rita texten
			g.setFont(new Font("Dialog", Font.BOLD, 12));
			FontMetrics fm = g.getFontMetrics();
			fm.stringWidth(label);
			g.setColor(Color.black);
			int x = (w - fm.stringWidth(label)) / 2;
			int y = (h - fm.getAscent() - fm.getDescent()) / 2 + fm.getAscent();
			g.drawString(label, x, y);
		} else { // Advanced mode
			g.setAntiAliasing(false);

			WfImage image = getResource(buttonImageFilenames[buttonState])
				.getWfImage();
			int xpos = (w - image.getWidth()) / 2;
			int ypos = (h - image.getHeight()) / 2;
			g.drawWfImage(image, xpos, ypos);

			if (!textImageFilenames[buttonState].equals("")) {
				WfImage textImage = getResource(textImageFilenames[buttonState])
					.getWfImage();
				xpos = (w - textImage.getWidth()) / 2;
				ypos = (h - textImage.getHeight()) / 2;
				g.drawWfImage(textImage, xpos, ypos);
			}
		}
	}

	/**
	 * Handles in-port events.
	 */
	public void processInportEvent(Wob arg, int inport) {
		if (inport == 3) { // Set Label
			label = ((wobs.text.player.TextWob) arg).getValue();
			repaint();
		} else if (inport == 4) { // Enable
			disabled = false;
			repaint();
		} else if (inport == 5) { // Disable
			disabled = true;
			repaint();
		}
		// Call super.processInportEvent() so that events sent to to default ports
		// are also handled.
		super.processInportEvent(arg, inport);
	}

	public void readData(DataInputStream datainputstream) {
		try {
			repeatRate = datainputstream.readDouble();
			basicMode = datainputstream.readBoolean();
			if (basicMode) {
				label = datainputstream.readUTF();
			} else {
				for (int i = 0; i < 4; i++) {
					// TODO: Use resource id instead of filename for resource fetching
					buttonImageFilenames[i] = datainputstream.readUTF();
					textImageFilenames[i] = datainputstream.readUTF();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}