/*
 * @(#)ButtonWob.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.slider.player;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import wobs.number.player.*;
import com.wirefusion.player.*;

/**
 * <p>
 * This class creates a slider Widget. It can display a slider horizontally
 * or vertically. It supports tick spacing.
 */

public class SliderWob extends Wob {
	/**
	 * True if horizontal slider, otherwise false
	 */
	private boolean isHorizontal;

	private double minValue = 0;

	private double maxValue = 100;

	/**
	 * Pixels between each tick, or 0 if tick spacing not used.
	 */
	private double tickSpacing = 0;

	/**
	 * The current slider value
	 */
	private double currentValue;

	private WfImage sliderImage = null;

	private WfImage markerImage = null;

	public SliderWob() {
	}

	public void init() {	
		enableEvents(AWTEvent.MOUSE_EVENT_MASK + AWTEvent.MOUSE_MOTION_EVENT_MASK);
		sendNumber(currentValue, 1);
	}
	
	public void processMouseEvent(MouseEvent ev) {
		if (ev.getID() == MouseEvent.MOUSE_CLICKED)
			setMarkerPosition(ev.getX(), ev.getY());
	}

	public void processMouseMotionEvent(MouseEvent ev) {
		if (ev.getID() == MouseEvent.MOUSE_DRAGGED)
			setMarkerPosition(ev.getX(), ev.getY());		
	}
	
	/**
	 * Sets the current slider value. Adjusts the value if it is below min, above
	 * max or not on a tick (if tick spacing used).
	 * 
	 * @param value
	 *          a value
	 */
	private void setValue(double value) {
		if (minValue < maxValue)
			value = value <= maxValue ? value >= minValue ? value : minValue : maxValue;
		else
			value = value <= minValue ? value >= maxValue ? value : maxValue : minValue;
		if (tickSpacing != 0) {
			if (minValue < maxValue)			
				value = value + tickSpacing/2;
			else
				value = value - tickSpacing/2;				
			value = value - ((value-minValue)%tickSpacing);			
		}			
		currentValue = value;		
	}	
	
	/**
	 * Sets the marker position when a mouse event has occured at mouseX, mouseY
	 * 
	 * @param mouseX an x position
	 * @param mouseY an y position
	 */
	private void setMarkerPosition(int mouseX, int mouseY) {
		// Set value if click or drag on slider
		if (isHorizontal) {
			int markerWidth = markerImage.getWidth();
			int markerX = mouseX - markerWidth / 2;
			if (markerX <= 0)
				markerX = 0;
			else if (markerX >= getSize().width - markerWidth)
				markerX = getSize().width - markerWidth;
			double value = minValue + ((double) markerX * (maxValue - minValue))
					/ (double) (getSize().width - markerWidth);
			setValue(value);
		} else {
			int markerHeight = markerImage.getHeight();
			int markerY = mouseY - markerHeight / 2;
			if (markerY <= 0)
				markerY = 0;
			else if (markerY >= getSize().height - markerHeight)
				markerY = getSize().height - markerHeight;
			double value = maxValue - ((double) markerY * (maxValue - minValue))
					/ (double) (getSize().height - markerHeight);
			setValue(value);
		}
		sendNumber(currentValue, 1);
		repaint();
	}
	
	private void loadImages() {
		if (sliderImage == null)
			sliderImage = getResourceById("1").getWfImage();
		if (markerImage == null)
			markerImage = getResourceById("2").getWfImage();
	}

	/**
	 * Paints slider and marker images.
	 * 
	 * If target area is wider or higher than slider image, then paint left part +
	 * right part + stretch the middle part in between so that it fits the target
	 * area width. If, instead, the slider image is wider or higher than the
	 * target area, then cut out the middle part of the slider
	 */
	public void paint(Graphics gr) {
		WfGraphics g = (WfGraphics)gr;
		loadImages();
		
		// Paint slider graphics
		if (isHorizontal) {
			double sliderY = (getSize().height - sliderImage.getHeight())/2d;
			double sliderBottomY = sliderY + sliderImage.getHeight();
			if (sliderImage.getWidth() == getSize().width) {
				g.drawWfImage(sliderImage, 0, sliderY);				
			} else { 
				// Slider target area has four key positions along x	
				// |-----------------------|
				// |1____|2__________|3____|4
				// |_____|___________|_____|
				//int x1 = 0;
				// x2 is smaller than half the slider image width, if target area has 
				// smaller with than slider image
				int x2 = Math.min(getSize().width/2, sliderImage.getWidth()/2);
				int x3 = getSize().width - x2;
				int x4 = getSize().width;
				// Make sure x3==x2 if middle section is not used, to avoid a "hole"
				if (getSize().width < sliderImage.getWidth()) 
					x3 = x2;
				
				// Draw left part of slider
				g.drawWfImage(sliderImage, 
						0, sliderY, x2, sliderBottomY, 
					  0, 0, x2, sliderImage.getHeight(), 100);
				
				// Draw right part of slider
				g.drawWfImage(sliderImage, 
						x3, sliderY, x4, sliderBottomY, 
					  sliderImage.getWidth() - (x4 - x3), 0, sliderImage.getWidth(), sliderImage.getHeight(), 100);
				
				// If target area is wider than slider image, draw middle part of slider
				if (getSize().width > sliderImage.getWidth()) {
					g.drawWfImage(sliderImage, 
							x2, sliderY, x3, sliderBottomY, 
						  sliderImage.getWidth()/2d, 0, 1+sliderImage.getWidth()/2d, sliderImage.getHeight(), 100);
				}
			}
		} else {
			double sliderX = (getSize().width - sliderImage.getWidth())/2d;
			double sliderRightX = sliderX + sliderImage.getWidth();
			if (sliderImage.getHeight() == getSize().height) {
				g.drawWfImage(sliderImage, sliderX, 0);
			} else {
				// Slider target area has four key positions along x
				//int y1 = 0;
				// x2 is smaller than half the slider image width, if target area has 
				// smaller with than slider image
				int y2 = Math.min(getSize().height/2, sliderImage.getHeight()/2);
				int y3 = getSize().height - y2;
				int y4 = getSize().height;
				// Make sure x3==x2 if middle section is not used, to avoid a "hole"
				if (getSize().height < sliderImage.getHeight()) 
					y3 = y2;
				
				// Draw top part of slider
				g.drawWfImage(sliderImage, 
						sliderX, 0, sliderRightX, y2,  
					  0, 0, sliderImage.getWidth(), 100, y2);
				
				// Draw right part of slider
				g.drawWfImage(sliderImage, 
						sliderX, y3, sliderRightX, y4, 
					  0, sliderImage.getHeight() - (y4 - y3), sliderImage.getWidth(), sliderImage.getHeight(), 100);
				
				// If target area is wider than slider image, draw middle part of slider
				if (getSize().height > sliderImage.getHeight()) {
					g.drawWfImage(sliderImage, 
							sliderX, y2, sliderRightX, y3,  
						  0, sliderImage.getHeight()/2d, sliderImage.getWidth(), 1+sliderImage.getHeight()/2d, 100);
				}
			}
		}

		// Paint marker
		int markerX = 0;
		int markerY = 0;
		if (isHorizontal) {
			markerX = (int) (((currentValue - minValue) * (double) (getSize().width - markerImage.getWidth())) / (maxValue - minValue));
			markerY = (getSize().height - markerImage.getHeight()) / 2;
		} else {
			markerX = (getSize().width - markerImage.getWidth()) / 2;
			markerY = (int) (((maxValue - currentValue) * (double) (getSize().height - markerImage.getHeight())) / (maxValue - minValue));
		}
		g.drawWfImage(markerImage, markerX, markerY);
	}

	public void processInportEvent(Wob arg, int inport) {
		if (inport == 1) { //set Slider Value 
			double value = ((NumberWob) arg).getValue();			
			setValue(value);
			sendNumber(currentValue, 1);
			repaint();
		} else if (inport == 2) { // Set Min Value
			double value = ((NumberWob) arg).getValue();				
			minValue = value;			
		} else if (inport == 3) { // Set Max Value
			double value = ((NumberWob) arg).getValue();				
			maxValue = value;			
		}
		super.processInportEvent(arg, inport);
	}

	public void readData(DataInputStream datainputstream) {
		try {
			isHorizontal = datainputstream.readBoolean();
			minValue = datainputstream.readDouble();
			maxValue = datainputstream.readDouble();
			double value = datainputstream.readDouble();
			setValue(value);			
			tickSpacing = datainputstream.readDouble();				
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

}